package net.intellisports.baro_service

import android.content.Context
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import android.view.Surface
import android.view.WindowManager
import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
/** BaroServicePlugin */
public class BaroServicePlugin: FlutterPlugin, MethodChannel.MethodCallHandler {
  private val METHOD_CHANNEL_NAME = "baro_service/method"
  private val BAROMETER_CHANNEL_NAME = "baro_service/barometer"

  private var barometerChannel: EventChannel? = null
  private var barometerStreamHandler: BarometerStreamHandler? = null
  private var sensorManager: SensorManager? = null

  private var methodChannel: MethodChannel? = null

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val plugin = BaroServicePlugin()
      plugin.setupEventChannel(registrar.context(), registrar.messenger())
    }
  }

  override fun onAttachedToEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    val context = binding.applicationContext
    setupEventChannel(context, binding.binaryMessenger)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
    when (call.method) {
      "isSensorAvailable" -> result.success(sensorManager!!.getSensorList(Sensor.TYPE_PRESSURE).isNotEmpty())
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      else -> result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    barometerChannel!!.setStreamHandler(null)
  }

  private fun setupEventChannel(context: Context, messenger: BinaryMessenger) {
    sensorManager = context.getSystemService(Context.SENSOR_SERVICE) as SensorManager

    methodChannel = MethodChannel(messenger, METHOD_CHANNEL_NAME)
    methodChannel!!.setMethodCallHandler(this)

    barometerChannel = EventChannel(messenger, BAROMETER_CHANNEL_NAME)
    barometerStreamHandler = BarometerStreamHandler(sensorManager!!, Sensor.TYPE_PRESSURE)
    barometerChannel!!.setStreamHandler(barometerStreamHandler)
  }
}

class BarometerStreamHandler(private val sensorManager: SensorManager, sensorType: Int, private var interval: Int = SensorManager.SENSOR_DELAY_NORMAL) :
        EventChannel.StreamHandler, SensorEventListener {
  private val sensor = sensorManager.getDefaultSensor(sensorType)
  private var eventSink: EventChannel.EventSink? = null

  override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
    if (sensor != null) {
      eventSink = events
      sensorManager.registerListener(this, sensor, interval)
    }
  }

  override fun onCancel(arguments: Any?) {
    sensorManager.unregisterListener(this)
    eventSink = null
  }

  override fun onAccuracyChanged(sensor: Sensor?, accuracy: Int) {

  }


  override fun onSensorChanged(event: SensorEvent?) {

    val unwrappedEvent: SensorEvent = event?.let { it } ?: return

    eventSink?.success(unwrappedEvent.values[0])

  }
}
