import Flutter
import UIKit
import CoreMotion

public class SwiftBaroServicePlugin: NSObject, FlutterPlugin {
    private let barometerStreamHandler = BarometerStreamHandler()
    
    init(registrar: FlutterPluginRegistrar) {
        let barometerChannel = FlutterEventChannel(name: "baro_service/barometer", binaryMessenger: registrar.messenger())
        barometerChannel.setStreamHandler(barometerStreamHandler)
    }
    
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "baro_service/method", binaryMessenger: registrar.messenger())
        let instance = SwiftBaroServicePlugin(registrar: registrar)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "isSensorAvailable":
            result(isSensorAvailable())
        case "getPlatformVersion":
            result("iOS " + UIDevice.current.systemVersion)
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    public func isSensorAvailable() -> Bool {
        return CMAltimeter.isRelativeAltitudeAvailable()
    }
    
}

class BarometerStreamHandler: NSObject, FlutterStreamHandler {
    private let altimeter = CMAltimeter()
    
    func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        if CMAltimeter.isRelativeAltitudeAvailable() {
            altimeter.startRelativeAltitudeUpdates(to: OperationQueue.main) { (data, error) in
                if data != nil {
                    events(data!.pressure.floatValue * 10) //pHa
                }
            }
        }
        return nil
    }
    
    func onCancel(withArguments arguments: Any?) -> FlutterError? {
        altimeter.stopRelativeAltitudeUpdates()
        return nil
    }
}
