import 'dart:async';
import 'package:flutter/services.dart';

final BaroService baroSersor = BaroService();
const MethodChannel _methodChannel = MethodChannel('baro_service/method');
const EventChannel _barometerChannel = EventChannel('baro_service/barometer');

class BarometerEvent {
  BarometerEvent(this.pressure);

  final double? pressure;

  @override
  String toString() => '[Current pressure (pHa: $pressure)]';
}

class BaroService {
  Stream<BarometerEvent>? _barometerEvents;

  Future<bool> isSensorAvailable() async {
    final available = await _methodChannel.invokeMethod('isSensorAvailable');
    return available;
  }

  Future<bool> isBarometerAvailable() => isSensorAvailable();

  Stream<BarometerEvent> get absolutePressure {
    if (_barometerEvents == null) {
      _barometerEvents = _barometerChannel
          .receiveBroadcastStream()
          .map((dynamic event) => BarometerEvent(event as double));
    }
    return _barometerEvents!;
  }

  static Future<String?> get platformVersion async {
    final String? version =
        await _methodChannel.invokeMethod('getPlatformVersion');
    return version;
  }
}
